use std::path::PathBuf;

use super::Element;
use super::GetStandardProperties;
use super::Object;
use super::StandardProperties;

pub type PriorityCookie = u8;
pub type HeadlineLevel = u16;

#[derive(Debug)]
pub struct Document<'s> {
    pub source: &'s str,
    pub category: Option<String>,
    pub path: Option<PathBuf>,
    pub zeroth_section: Option<Section<'s>>,
    pub children: Vec<Heading<'s>>,
}

#[derive(Debug)]
pub struct Heading<'s> {
    pub source: &'s str,
    pub level: HeadlineLevel,
    pub todo_keyword: Option<(TodoKeywordType, &'s str)>,
    pub priority_cookie: Option<PriorityCookie>,
    pub title: Vec<Object<'s>>,
    pub tags: Vec<&'s str>,
    pub children: Vec<DocumentElement<'s>>,
    pub is_comment: bool,
    pub is_archived: bool,
    pub is_footnote_section: bool,
}

#[derive(Debug)]
pub struct Section<'s> {
    pub source: &'s str,
    pub children: Vec<Element<'s>>,
}

#[derive(Debug)]
pub enum DocumentElement<'s> {
    Heading(Heading<'s>),
    Section(Section<'s>),
}

#[derive(Debug)]
pub enum TodoKeywordType {
    Todo,
    Done,
}

impl<'s> GetStandardProperties<'s> for DocumentElement<'s> {
    fn get_standard_properties<'b>(&'b self) -> &'b dyn StandardProperties<'s> {
        match self {
            DocumentElement::Heading(inner) => inner,
            DocumentElement::Section(inner) => inner,
        }
    }
}

impl<'s> StandardProperties<'s> for Document<'s> {
    fn get_source<'b>(&'b self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Section<'s> {
    fn get_source<'b>(&'b self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Heading<'s> {
    fn get_source<'b>(&'b self) -> &'s str {
        self.source
    }
}

impl<'s> Heading<'s> {
    pub fn get_raw_value(&self) -> String {
        // TODO: I think this could just return a string slice instead of an owned string.
        let title_source: String = self
            .title
            .iter()
            .map(|obj| obj.get_standard_properties().get_source())
            .collect();
        title_source
    }
}
